﻿using static Celeste.Mod.KelperApiExample.KelperImports;
using Microsoft.Xna.Framework;
using Monocle;
using System;
using Celeste.Mod.Entities;

namespace Celeste.Mod.KelperApiExample.Entities;

[CustomEntity("KelperApiExample/Ball")]
public class NailCompatibleBall : Actor
{
    public Sprite sprite;
    public Vector2 Speed;

    public NailCompatibleBall(EntityData data, Vector2 offset)
        : base(data.Position + offset)
    {
        base.Depth = 100;
        base.Collider = new Hitbox(8f, 10f, -4f, -10f);
        base.Tag = Tags.TransitionUpdate;
        Add(sprite = GFX.SpriteBank.Create("badelineBoost"));
        sprite.Position.Y -= 6;
        Add(new VertexLight(base.Collider.Center, Color.White, 1f, 32, 64));
          

        float knockbackStrength = 300f;
        Hitbox nailHitbox = new Hitbox(10f, 10f, -5f, -10f);
        object[] parameters = new object[] { this, knockbackStrength };

        // You can also write the parameters directly with collection expressions
        // Component nailCollider = CreateNailColliderWithCustomHitbox?.Invoke(OnNailHit, nailHitbox, [this, knockbackStrength]));
        Component nailCollider = CreateNailColliderWithCustomHitbox?.Invoke(OnNailHit, nailHitbox, parameters);
        if (nailCollider != null) Add(nailCollider);
    }

    // The player entity and the absolute nail direction are always the first 2 parameters.
    // They are followed by the user defined parameters.
    private static void OnNailHit(Player player, Vector2 nailDir, NailCompatibleBall self, float knockbackStrength)
    {
        // Remember the null-conditional operator when invoking the delegate  -->  ?
        // It stops the code from executing if the delegate is null.
        // Without it the code will crash if kelper is not loaded.

        // Since this method is only ran when a nail hit happens, the null-conditional operator isn't necessary because kelper has to be loaded.
        // I still suggest using it though.
        // It is mandatory in the constructor!

        // The parameter nailDir is normally relative to the screen. Upswing is always Vector2(0, -1). 
        // Here we get a nailDir that is relative to the players gravitational orientation.

        // If the nailDir is down, (up when GravityHelper has flipped gravity), we call the NailRebound method.
        if (GetGravityRelativeNailDir.Invoke().Y > 0)
            // Uses kelpers nailrebound method to knock the player back.
            ApplyNailRebound?.Invoke(1.3f);

        // Plays a tink sound on the player's position.
        PlayNailTinkSound?.Invoke();

        // Removes the nail hitbox for the current swing.
        ConsumeNailSwing?.Invoke();


        self.Speed = nailDir * knockbackStrength;
    }


    // Just some movement logig ripped from TheoCrystal.
    // Not a part of this example, just something to make the entity do something slightly interesting.
    public override void Update()
    {
        base.Update();

        if (OnGround())
        {
            float target = ((!OnGround(Position + Vector2.UnitX * 3f)) ? 20f : (OnGround(Position - Vector2.UnitX * 3f) ? 0f : (-20f)));
            Speed.X = Calc.Approach(Speed.X, target, 800f * Engine.DeltaTime);
        }
        else
        {
            float num = 800f;
            if (Math.Abs(Speed.Y) <= 30f)
            {
                num *= 0.5f;
            }

            float num2 = 350f;
            if (Speed.Y < 0f)
            {
                num2 *= 0.5f;
            }

            Speed.X = Calc.Approach(Speed.X, 0f, num2 * Engine.DeltaTime);
            Speed.Y = Calc.Approach(Speed.Y, 200f, num * Engine.DeltaTime);
        }

        MoveH(Speed.X * Engine.DeltaTime);
        MoveV(Speed.Y * Engine.DeltaTime);
    }

}

