﻿//                                                                                          //
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>//
//                                                                                          //
//                                                                                          //
//                You can copy and paste this file directly into your mod                   //
//            You only need to change the namespace Celeste.Mod.KelperApiExample            //
//                            to match your own mod's namespace                             //
//               and add KelperImports.Load() into your module's Load function              //
//                                                                                          //
//                                                                                          //
//<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<//
//                                                                                          //


using Microsoft.Xna.Framework;
using MonoMod.ModInterop;
using Monocle;
using System;

namespace Celeste.Mod.KelperApiExample;

[ModImportName("KelperAPI")]
public class KelperImports
{
    public static void Load()
    {
        typeof(KelperImports).ModInterop();
    }

    /// <summary>
    ///     If there is some massive change that breaks the modinterop this version number will change.  <br/>
    ///     It should never happen hopefully.
    /// </summary>
    public static Func<int> GetVersion;


    /// <summary>
    ///     Returns a KelperNailCollider as a component                                                 <br/><br/>
    ///     
    ///     <b>Delegate</b> - The function that get's called when the entity is hit with the nail. 
    ///     The first two parameters are always Player player and Vector2 nailDir                       <br/>
    ///     <b>object[]</b> - Optional array of parameters that the delegate function will use. They come after the 2 mandatory parameters.  <br/><br/>
    ///     
    /// For example the following collider calls the following method:  <br/>
    /// <code>
    ///     Add(CreateNailCollider?.Invoke(OnNailHit, [10, "something", new Vector2(1, 20)])); <br/>
    ///     public static void OnNailHit(Player player, Vector2 nailDir, int someNumber, string someText, Vector2 someVector)
    /// </code>
    /// </summary>
    /// <returns>
    ///     A KelperNailCollider as a component
    /// </returns>
    public static Func<Delegate, object[], Component> CreateNailCollider;


    /// <summary>
    ///     Returns a KelperNailCollider as a component with a custom nail hitbox                       <br/><br/>
    ///     
    ///     <b>Delegate</b> - The function that get's called when the entity is hit with the nail. 
    ///     The first two parameters are always Player player and Vector2 nailDir                       <br/>
    ///     <b>Hitbox</b> - A hitbox that the nail must hit for the Delegate to be invoked.             <br/>
    ///     <b>object[]</b> - Optional array of parameters that the delegate function will use. They come after the 2 mandatory parameters.  <br/>
    ///     
    /// </summary>
    /// <returns>
    ///     A KelperNailCollider as a component
    /// </returns>
    public static Func<Delegate, Hitbox, object[], Component> CreateNailColliderWithCustomHitbox;


    /// <summary>
    /// Returns a nailDir that is flipped vertically if the player is has upside down gravity           <br/><br/>
    /// 
    /// Vector2(0, 1) means that the player is swinging towards their feet.
    /// 
    /// </summary>
    /// <returns>
    ///     Vector2
    /// </returns>
    public static Func<Vector2> GetGravityRelativeNailDir;


    /// <summary>
    ///     Bounces the player backwards relative to the nail swing direction                           <br/><br/>
    ///     
    ///     <b>Float</b> - Rebound strength multiplier. Minimum value is 1f                             <br/><br/>
    ///     
    ///     If multiple entities are hit on the same frame, the highest multiplier will be applied.     <br/>
    ///     Has all logic to handle adding speed and backboosting etc.
    /// </summary>
    public static Action<float> ApplyNailRebound;


    /// <summary>
    ///     Plays a kind of metallic tink sound
    /// </summary>
    public static Action PlayNailTinkSound;


    /// <summary>
    /// Disables the nail hitbox for the current swing. All entities hit on this frame will still be hit.   <br/><br/>
    /// </summary>
    public static Action ConsumeNailSwing;

}
